@@ -2,9 +2,17 @@ from pymongo import MongoClient, ReadPreference, uri_parser
 from pymongo.database import _check_name
 import six
 
-__all__ = ['MongoEngineConnectionError', 'connect', 'disconnect', 'disconnect_all',
-           'register_connection', 'DEFAULT_CONNECTION_NAME', 'DEFAULT_DATABASE_NAME',
-           'get_db', 'get_connection']
+__all__ = [
+    'DEFAULT_CONNECTION_NAME',
+    'DEFAULT_DATABASE_NAME',
+    'MongoEngineConnectionError',
+    'connect',
+    'disconnect',
+    'disconnect_all',
+    'get_connection',
+    'get_db',
+    'register_connection',
+]
 
 
 DEFAULT_CONNECTION_NAME = 'default'
@@ -263,17 +271,17 @@ def get_connection(alias=DEFAULT_CONNECTION_NAME, reconnect=False):
     else:
         connection_class = MongoClient
 
-    # Re-use existing connection if one is suitable
+    # Re-use existing connection if one is suitable.
     existing_connection = _find_existing_connection(raw_conn_settings)
-
-    # If an existing connection was found, assign it to the new alias
     if existing_connection:
-        _connections[alias] = existing_connection
+        connection = existing_connection
     else:
-        _connections[alias] = _create_connection(alias=alias,
-                                                 connection_class=connection_class,
-                                                 **conn_settings)
-
+        connection = _create_connection(
+            alias=alias,
+            connection_class=connection_class,
+            **conn_settings
+        )
+    _connections[alias] = connection
     return _connections[alias]
 
 
@@ -359,8 +367,11 @@ def connect(db=None, alias=DEFAULT_CONNECTION_NAME, **kwargs):
         new_conn_settings = _get_connection_settings(db, **kwargs)
 
         if new_conn_settings != prev_conn_setting:
-            raise MongoEngineConnectionError(
-                'A different connection with alias `%s` was already registered. Use disconnect() first' % alias)
+            err_msg = (
+                'A different connection with alias `{}` was already '
+                'registered. Use disconnect() first'
+            ).format(alias)
+            raise MongoEngineConnectionError(err_msg)
     else:
         register_connection(alias, db, **kwargs)
 
